@@ -48,6 +48,16 @@
 
 #endif
 
+struct PangoAlias
+{
+  char *alias;
+  int n_families;
+  char **families;
+  gboolean visible; /* Do we want/need this? */
+};
+
+GHashTable *pango_aliases_ht = NULL;
+
 /**
  * pango_trim_string:
  * @str: a string
@@ -557,7 +567,7 @@ read_config_file (const char *filename, gboolean enoent_error)
     }
       
   if (ferror (file))
-    errstring = g_strdup ("g_strerror(errno)");
+    errstring = g_strdup (g_strerror(errno));
   
  error:
 
@@ -626,37 +636,17 @@ pango_config_key_get (const char *key)
   return g_strdup (g_hash_table_lookup (config_hash, key));
 }
 
-#ifdef G_OS_WIN32
-
 /* DllMain function needed to tuck away the DLL name */
 
-static char dll_name[MAX_PATH];
+G_WIN32_DLLMAIN_FOR_DLL_NAME(static, dll_name)
 
-BOOL WINAPI
-DllMain (HINSTANCE hinstDLL,
-	 DWORD     fdwReason,
-	 LPVOID    lpvReserved)
-{
-  switch (fdwReason)
-    {
-    case DLL_PROCESS_ATTACH:
-      GetModuleFileName ((HMODULE) hinstDLL, dll_name, sizeof (dll_name));
-      break;
-    }
-
-  return TRUE;
-}
-
-#endif /* G_OS_WIN32 */
 
 /**
  * pango_get_sysconf_subdirectory:
  *
  * On Unix, returns the name of the "pango" subdirectory of SYSCONFDIR
- * (which is set at compile time). On Win32, returns the Pango
- * installation directory (which is set at installation time, and
- * stored in the registry). The returned string should not be
- * freed.
+ * (which is set at compile time). On Win32, returns a subdirectory of
+ * the Pango installation directory.
  *
  * Return value: the Pango sysconf directory. The returned string should
  * not be freed. 
@@ -669,7 +659,7 @@ pango_get_sysconf_subdirectory (void)
 
   if (result == NULL)
     result = g_win32_get_package_installation_subdirectory
-      (PACKAGE " " VERSION, g_path_get_basename (dll_name), "etc\\pango");
+      (PACKAGE " " VERSION, dll_name, "etc\\pango");
 
   return result;
 #else
@@ -697,7 +687,7 @@ pango_get_lib_subdirectory (void)
 
   if (result == NULL)
     result = g_win32_get_package_installation_subdirectory
-      (PACKAGE " " VERSION, g_path_get_basename (dll_name), "lib\\pango");
+      (PACKAGE " " VERSION, dll_name, "lib\\pango");
 
   return result;
 #else
@@ -729,21 +719,21 @@ pango_parse_style (const char *str,
     {
     case 'n':
     case 'N':
-      if (g_strcasecmp (str, "normal") == 0)
+      if (g_ascii_strcasecmp (str, "normal") == 0)
 	{
 	  *style = PANGO_STYLE_NORMAL;
 	  return TRUE;
 	}
       break;
     case 'i':
-      if (g_strcasecmp (str, "italic") == 0)
+      if (g_ascii_strcasecmp (str, "italic") == 0)
 	{
 	  *style = PANGO_STYLE_ITALIC;
 	  return TRUE;
 	}
       break;
     case 'o':
-      if (g_strcasecmp (str, "oblique") == 0)
+      if (g_ascii_strcasecmp (str, "oblique") == 0)
 	{
 	  *style = PANGO_STYLE_OBLIQUE;
 	  return TRUE;
@@ -780,7 +770,7 @@ pango_parse_variant (const char   *str,
     {
     case 'n':
     case 'N':
-      if (g_strcasecmp (str, "normal") == 0)
+      if (g_ascii_strcasecmp (str, "normal") == 0)
 	{
 	  *variant = PANGO_VARIANT_NORMAL;
 	  return TRUE;
@@ -788,8 +778,8 @@ pango_parse_variant (const char   *str,
       break;
     case 's':
     case 'S':
-      if (g_strcasecmp (str, "small_caps") == 0 ||
-	  g_strcasecmp (str, "smallcaps") == 0)
+      if (g_ascii_strcasecmp (str, "small_caps") == 0 ||
+	  g_ascii_strcasecmp (str, "smallcaps") == 0)
 	{
 	  *variant = PANGO_VARIANT_SMALL_CAPS;
 	  return TRUE;
@@ -826,7 +816,7 @@ pango_parse_weight (const char  *str,
     {
     case 'b':
     case 'B':
-      if (g_strcasecmp (str, "bold") == 0)
+      if (g_ascii_strcasecmp (str, "bold") == 0)
 	{
 	  *weight = PANGO_WEIGHT_BOLD;
 	  return TRUE;
@@ -834,7 +824,7 @@ pango_parse_weight (const char  *str,
       break;
     case 'h':
     case 'H':
-      if (g_strcasecmp (str, "heavy") == 0)
+      if (g_ascii_strcasecmp (str, "heavy") == 0)
 	{
 	  *weight = PANGO_WEIGHT_HEAVY;
 	  return TRUE;
@@ -842,7 +832,7 @@ pango_parse_weight (const char  *str,
       break;
     case 'l':
     case 'L':
-      if (g_strcasecmp (str, "light") == 0)
+      if (g_ascii_strcasecmp (str, "light") == 0)
 	{
 	  *weight = PANGO_WEIGHT_LIGHT;
 	  return TRUE;
@@ -850,7 +840,7 @@ pango_parse_weight (const char  *str,
       break;
     case 'n':
     case 'N':
-      if (g_strcasecmp (str, "normal") == 0)
+      if (g_ascii_strcasecmp (str, "normal") == 0)
 	{
 	  *weight = PANGO_WEIGHT_NORMAL;
 	  return TRUE;
@@ -858,12 +848,12 @@ pango_parse_weight (const char  *str,
       break;
     case 'u':
     case 'U':
-      if (g_strcasecmp (str, "ultralight") == 0)
+      if (g_ascii_strcasecmp (str, "ultralight") == 0)
 	{
 	  *weight = PANGO_WEIGHT_ULTRALIGHT;
 	  return TRUE;
 	}
-      else if (g_strcasecmp (str, "ultrabold") == 0)
+      else if (g_ascii_strcasecmp (str, "ultrabold") == 0)
 	{
 	  *weight = PANGO_WEIGHT_ULTRABOLD;
 	  return TRUE;
@@ -924,7 +914,7 @@ pango_parse_stretch (const char   *str,
     { 
     case 'c':
     case 'C':
-      if (g_strcasecmp (str, "condensed") == 0)
+      if (g_ascii_strcasecmp (str, "condensed") == 0)
 	{
 	  *stretch = PANGO_STRETCH_CONDENSED;
 	  return TRUE;
@@ -932,19 +922,19 @@ pango_parse_stretch (const char   *str,
       break;
     case 'e':
     case 'E':
-      if (g_strcasecmp (str, "extra_condensed") == 0 ||
-	  g_strcasecmp (str, "extracondensed") == 0)
+      if (g_ascii_strcasecmp (str, "extra_condensed") == 0 ||
+	  g_ascii_strcasecmp (str, "extracondensed") == 0)
 	{
 	  *stretch = PANGO_STRETCH_EXTRA_CONDENSED;
 	  return TRUE;
 	}
-     if (g_strcasecmp (str, "extra_expanded") == 0 ||
-	 g_strcasecmp (str, "extraexpanded") == 0)
+     if (g_ascii_strcasecmp (str, "extra_expanded") == 0 ||
+	 g_ascii_strcasecmp (str, "extraexpanded") == 0)
 	{
 	  *stretch = PANGO_STRETCH_EXTRA_EXPANDED;
 	  return TRUE;
 	}
-      if (g_strcasecmp (str, "expanded") == 0)
+      if (g_ascii_strcasecmp (str, "expanded") == 0)
 	{
 	  *stretch = PANGO_STRETCH_EXPANDED;
 	  return TRUE;
@@ -952,7 +942,7 @@ pango_parse_stretch (const char   *str,
       break;
     case 'n':
     case 'N':
-      if (g_strcasecmp (str, "normal") == 0)
+      if (g_ascii_strcasecmp (str, "normal") == 0)
 	{
 	  *stretch = PANGO_STRETCH_NORMAL;
 	  return TRUE;
@@ -960,14 +950,14 @@ pango_parse_stretch (const char   *str,
       break;
     case 's':
     case 'S':
-      if (g_strcasecmp (str, "semi_condensed") == 0 ||
-	  g_strcasecmp (str, "semicondensed") == 0)
+      if (g_ascii_strcasecmp (str, "semi_condensed") == 0 ||
+	  g_ascii_strcasecmp (str, "semicondensed") == 0)
 	{
 	  *stretch = PANGO_STRETCH_SEMI_CONDENSED;
 	  return TRUE;
 	}
-      if (g_strcasecmp (str, "semi_expanded") == 0 ||
-	  g_strcasecmp (str, "semiexpanded") == 0)
+      if (g_ascii_strcasecmp (str, "semi_expanded") == 0 ||
+	  g_ascii_strcasecmp (str, "semiexpanded") == 0)
 	{
 	  *stretch = PANGO_STRETCH_SEMI_EXPANDED;
 	  return TRUE;
@@ -975,14 +965,14 @@ pango_parse_stretch (const char   *str,
       break;
     case 'u':
     case 'U':
-      if (g_strcasecmp (str, "ultra_condensed") == 0 ||
-	  g_strcasecmp (str, "ultracondensed") == 0)
+      if (g_ascii_strcasecmp (str, "ultra_condensed") == 0 ||
+	  g_ascii_strcasecmp (str, "ultracondensed") == 0)
 	{
 	  *stretch = PANGO_STRETCH_ULTRA_CONDENSED;
 	  return TRUE;
 	}
-      if (g_strcasecmp (str, "ultra_expanded") == 0 ||
-	  g_strcasecmp (str, "ultraexpanded") == 0)
+      if (g_ascii_strcasecmp (str, "ultra_expanded") == 0 ||
+	  g_ascii_strcasecmp (str, "ultraexpanded") == 0)
 	{
 	  *stretch = PANGO_STRETCH_ULTRA_EXPANDED;
 	  return TRUE;
@@ -1072,7 +1062,7 @@ pango_language_get_type (void)
  * pointer) and compared with other language tags (compare the
  * pointer.)
  *
- * This function first canonicalizes the string by by converting it to
+ * This function first canonicalizes the string by converting it to
  * lowercase, mapping '_' to '-', and stripping all characters other
  * than letters and '-'.
  * 
@@ -1270,3 +1260,235 @@ pango_get_mirror_char (gunichar        ch,
 }
 
 #endif /* HAVE_FRIBIDI */
+
+
+guint
+alias_hash (struct PangoAlias *alias)
+{
+  return g_str_hash (alias->alias);
+}
+
+gboolean
+alias_equal (struct PangoAlias *alias1,
+	     struct PangoAlias *alias2)
+{
+  return g_str_equal (alias1->alias,
+		      alias2->alias);
+}
+
+
+void
+alias_free (struct PangoAlias *alias)
+{
+  int i;
+  g_free (alias->alias);
+
+  for (i = 0; i < alias->n_families; i++)
+    g_free (alias->families[i]);
+
+  g_free (alias->families);
+  
+  g_free (alias);
+}
+
+static void
+read_alias_file (const char *filename)
+{
+  FILE *file;
+    
+  GString *line_buffer;
+  GString *tmp_buffer1;
+  GString *tmp_buffer2;
+  char *errstring = NULL;
+  const char *pos;
+  int line = 0;
+  struct PangoAlias alias_key;
+  struct PangoAlias *alias;
+  char **new_families;
+  int n_new;
+  int i;
+
+  file = fopen (filename, "r");
+  if (!file)
+    return;
+
+  line_buffer = g_string_new (NULL);
+  tmp_buffer1 = g_string_new (NULL);
+  tmp_buffer2 = g_string_new (NULL);
+
+  while (pango_read_line (file, line_buffer))
+    {
+      gboolean empty = FALSE;
+      gboolean append = FALSE;
+      line++;
+
+      pos = line_buffer->str;
+      if (!pango_skip_space (&pos))
+	continue;
+      
+      if (!pango_scan_word (&pos, tmp_buffer1) ||
+	  !pango_skip_space (&pos))
+	{
+	  errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
+	  goto error;
+	}
+      
+      if (*pos == '+')
+	{
+	  append = TRUE;
+	  pos++;
+	}
+
+      if (*(pos++) != '=')
+	{
+	  errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
+	  goto error;
+	}
+      
+      if (!pango_skip_space (&pos))
+	{
+	  empty = TRUE;
+	}
+      else
+	{
+	  if (!pango_scan_string (&pos, tmp_buffer2))
+	    {
+	      errstring = g_strdup ("Error parsing value string");
+	      goto error;
+	    }
+	  if (pango_skip_space (&pos))
+	    {
+	      errstring = g_strdup ("Junk after value string");
+	      goto error;
+	    }
+	}
+
+      alias_key.alias = g_ascii_strdown (tmp_buffer1->str, -1);
+      
+      /* Remove any existing values */
+      alias = g_hash_table_lookup (pango_aliases_ht, &alias_key);
+      
+      if (!alias)
+	{
+	  alias = g_new0 (struct PangoAlias, 1);
+	  alias->alias = alias_key.alias;
+	  
+	  g_hash_table_insert (pango_aliases_ht,
+			       alias, alias);
+	}
+      else
+	g_free (alias_key.alias);
+	  
+      new_families = g_strsplit (tmp_buffer2->str, ",", -1);
+      
+      n_new = 0;
+      while (new_families[n_new])
+	n_new++;
+      
+      if (alias->families && append)
+	{
+	  alias->families = g_realloc (alias->families,
+				       sizeof (char *) *(n_new + alias->n_families));
+	  for (i = 0; i < n_new; i++)
+	    alias->families[alias->n_families + i] = new_families[i];
+	  g_free (new_families);
+	  alias->n_families += n_new;
+	}
+      else
+	{
+	  for (i = 0; i < alias->n_families; i++)
+	    g_free (alias->families[i]);
+	  g_free (alias->families);
+	  
+	  alias->families = new_families;
+	  alias->n_families = n_new;
+	}
+    }
+
+  if (ferror (file))
+    errstring = g_strdup (g_strerror(errno));
+  
+ error:
+
+  if (errstring)
+    {
+      fprintf (stderr, "Pango:%s:%d: %s\n", filename, line, errstring);
+      g_free (errstring);
+    }
+      
+  g_string_free (line_buffer, TRUE);
+  g_string_free (tmp_buffer1, TRUE);
+  g_string_free (tmp_buffer2, TRUE);
+
+  fclose (file);
+}
+
+void
+pango_load_aliases (void)
+{
+  char *filename;
+  const char *home;
+
+  pango_aliases_ht = g_hash_table_new_full ((GHashFunc)alias_hash,
+					    (GEqualFunc)alias_equal,
+					    (GDestroyNotify)alias_free,
+					    NULL);
+
+
+  filename = g_strconcat (pango_get_sysconf_subdirectory (),
+			  G_DIR_SEPARATOR_S "pango.aliases",
+			  NULL);
+  read_alias_file (filename);
+  g_free (filename);
+  
+  home = g_get_home_dir ();
+  if (home && *home)
+    {
+      filename = g_strconcat (home,
+			      G_DIR_SEPARATOR_S ".pango.aliases",
+			      NULL);
+      read_alias_file (filename);
+      g_free (filename);
+    }
+}
+
+
+/**
+ * pango_lookup_aliases:
+ * @fontname: an ascii string
+ * @families: will be set to an array of font family names.
+ *    this array is owned by pango and should not be freed.
+ *
+ * Look up all user defined aliases for the alias #fontname.
+ * The resulting font family names will be stored in #families,
+ * and the number of families will be returned.
+ * 
+ * Return value: the number of font famillies stored in the #families argument.
+ *   This value is owned by Pango and must not be freed.
+ **/
+void
+pango_lookup_aliases (const char   *fontname,
+		      char       ***families,
+		      int          *n_families)
+{
+  struct PangoAlias alias_key;
+  struct PangoAlias *alias;
+  
+  if (pango_aliases_ht == NULL)
+    pango_load_aliases ();
+
+  alias_key.alias = g_ascii_strdown (fontname, -1);
+  alias = g_hash_table_lookup (pango_aliases_ht, &alias_key);
+  g_free (alias_key.alias);
+
+  if (alias)
+    {
+      *families = alias->families;
+      *n_families = alias->n_families;
+    }
+  else
+    {
+      *families = NULL;
+      *n_families = 0;
+    }
+}
