@@ -129,7 +129,7 @@ pango_win32_get_context (void)
     }
 
   result = pango_context_new ();
-  pango_context_add_font_map (result, pango_win32_font_map_for_display ());
+  pango_context_set_font_map (result, pango_win32_font_map_for_display ());
 
   return result;
 }
@@ -997,6 +997,15 @@ font_get_unicode_table (PangoFont *font)
   return table;
 }
 
+/**
+ * pango_win32_font_get_glyph_index:
+ * @font: a #PangoFont.
+ * @wc: a Unicode character.
+ *
+ * Obtains the index of the glyph for @wc in @font.
+ *
+ * Returns: the glyph index for @wc.
+ **/
 gint
 pango_win32_font_get_glyph_index (PangoFont *font,
 				  gunichar   wc)
