@@ -112,22 +112,21 @@ pango_xft_font_class_init (PangoXftFontClass *class)
 
 PangoXftFont *
 _pango_xft_font_new (PangoFontMap               *fontmap,
-		     const PangoFontDescription *description,
-		     XftFont                    *xft_font)
+		     XftPattern                 *pattern)
 {
   PangoXftFont *xfont;
 
   g_return_val_if_fail (fontmap != NULL, NULL);
-  g_return_val_if_fail (description != NULL, NULL);
-  g_return_val_if_fail (xft_font != NULL, NULL);
+  g_return_val_if_fail (pattern != NULL, NULL);
 
   xfont = (PangoXftFont *)g_object_new (PANGO_TYPE_XFT_FONT, NULL);
   
   xfont->fontmap = fontmap;
+  xfont->font_pattern = pattern;
   
   g_object_ref (G_OBJECT (fontmap));
-  xfont->description = pango_font_description_copy (description);
-  xfont->xft_font = xft_font;
+  xfont->description = _pango_xft_font_desc_from_pattern (pattern);
+  xfont->xft_font = NULL;
 
   _pango_xft_font_map_add (xfont->fontmap, xfont);
 
@@ -155,10 +154,10 @@ get_mini_font (PangoFont *font)
       pango_font_description_set_size (desc,
 				       0.5 * pango_font_description_get_size (xfont->description));
       
-      xfont->mini_font = pango_font_map_load_font (xfont->fontmap, desc);
+      xfont->mini_font = pango_font_map_load_font (xfont->fontmap, NULL, desc);
       pango_font_description_free (desc);
       
-      mini_xft = ((PangoXftFont *)xfont->mini_font)->xft_font;
+      mini_xft = pango_xft_font_get_font (xfont->mini_font);
       face = pango_xft_font_get_face (xfont->mini_font);
       
       for (i = 0 ; i < 16 ; i++)
@@ -181,7 +180,32 @@ get_mini_font (PangoFont *font)
 }
 
 static void
-draw_box (XftDraw      *draw,
+draw_rectangle (Display      *display,
+		Picture       src_picture,
+		Picture       dest_picture,
+		XftDraw      *draw,
+		XftColor     *color,
+		gint          x,
+		gint          y,
+		gint          width,
+		gint          height)
+{
+  if (draw)
+    {
+      XftDrawRect (draw, color, x, y, width, height);
+    }
+  else
+    {
+      XRenderComposite (display, PictOpOver, src_picture, None, dest_picture,
+			0, 0, 0, 0, x, y, width, height);
+    }
+}
+
+static void
+draw_box (Display      *display,
+	  Picture       src_picture,
+	  Picture       dest_picture,
+	  XftDraw      *draw,
 	  XftColor     *color,
 	  PangoXftFont *xfont,
 	  gint          x,
@@ -189,16 +213,117 @@ draw_box (XftDraw      *draw,
 	  gint          width,
 	  gint          height)
 {
-  XftDrawRect (draw, color,
-	       x, y, width, xfont->mini_pad);
-  XftDrawRect (draw, color,
-	       x, y + xfont->mini_pad, xfont->mini_pad, height - xfont->mini_pad * 2);
-  XftDrawRect (draw, color,
-	       x + width - xfont->mini_pad, y + xfont->mini_pad, xfont->mini_pad, height - xfont->mini_pad * 2);
-  XftDrawRect (draw, color,
-	       x, y + height - xfont->mini_pad, width, xfont->mini_pad);
+  draw_rectangle (display, src_picture, dest_picture, draw, color,
+		  x, y, width, xfont->mini_pad);
+  draw_rectangle (display, src_picture, dest_picture, draw, color,
+		  x, y + xfont->mini_pad, xfont->mini_pad, height - xfont->mini_pad * 2);
+  draw_rectangle (display, src_picture, dest_picture, draw, color,
+		  x + width - xfont->mini_pad, y + xfont->mini_pad, xfont->mini_pad, height - xfont->mini_pad * 2);
+  draw_rectangle (display, src_picture, dest_picture, draw, color,
+		  x, y + height - xfont->mini_pad, width, xfont->mini_pad);
 }
 
+/**
+ * pango_xft_render:
+ * @draw:    the XftDraw object.
+ * @color:   the color in which to draw the string
+ * @font:    the font in which to draw the string
+ * @glyphs:  the glyph string to draw
+ * @x:       the x position of start of string (in pixels)
+ * @y:       the y position of baseline (in pixels)
+ *
+ * Render a PangoGlyphString onto an XftDraw object wrapping an X drawable.
+ */
+static void
+pango_xft_real_render (Display          *display,
+		       Picture           src_picture,
+		       Picture           dest_picture,
+		       XftDraw          *draw,
+		       XftColor         *color,
+		       PangoFont        *font,
+		       PangoGlyphString *glyphs,
+		       gint              x,
+		       gint              y)
+{
+  PangoXftFont *xfont = PANGO_XFT_FONT (font);
+  XftFont *xft_font = pango_xft_font_get_font (font);
+  int i;
+  int x_off = 0;
+
+  /* Slow initial implementation. For speed, it should really
+   * collect the characters into runs, and draw multiple
+   * characters with each XftDrawString32 call.
+   */
+  if (!display)
+    _pango_xft_font_map_get_info (xfont->fontmap, &display, NULL);
+  
+  for (i=0; i<glyphs->num_glyphs; i++)
+    {
+      PangoGlyph glyph = glyphs->glyphs[i].glyph;
+
+      if (glyph)
+	{
+	  if (glyph & PANGO_XFT_UNKNOWN_FLAG)
+	    {
+	      char buf[5];
+	      int ys[3];
+	      int xs[3];
+	      int j, k;
+	      
+	      PangoFont *mini_font = get_mini_font (font);
+	      XftFont *mini_xft = pango_xft_font_get_font (mini_font);
+	      FT_Face face = pango_xft_font_get_face (xfont->mini_font);
+      
+	      glyph &= ~PANGO_XFT_UNKNOWN_FLAG;
+
+	      ys[0] = y + PANGO_PIXELS (glyphs->glyphs[i].geometry.y_offset) - xft_font->ascent + (xft_font->ascent + xft_font->descent - xfont->mini_height * 2 - xfont->mini_pad * 5) / 2;
+	      ys[1] = ys[0] + 2 * xfont->mini_pad + xfont->mini_height;
+	      ys[2] = ys[1] + xfont->mini_height + xfont->mini_pad;
+
+	      xs[0] = x + PANGO_PIXELS (x_off + glyphs->glyphs[i].geometry.x_offset); 
+	      xs[1] = xs[0] + 2 * xfont->mini_pad;
+	      xs[2] = xs[1] + xfont->mini_width + xfont->mini_pad;
+	      
+	      draw_box (display, src_picture, dest_picture, draw, color, xfont,
+			xs[0], ys[0],
+			xfont->mini_width * 2 + xfont->mini_pad * 5,
+			xfont->mini_height * 2 + xfont->mini_pad * 5);
+	      
+	      g_snprintf (buf, sizeof(buf), "%04X", glyph);
+
+	      for (j = 0; j < 2; j++)
+		for (k = 0; k < 2; k++)
+		  {
+		    XftChar32 glyph = FT_Get_Char_Index (face, buf[2*j + k]);
+		    if (draw)
+		      XftDrawString32 (draw, color, mini_xft,
+				       xs[k+1], ys[j+1], &glyph, 1);
+		    else
+		      XftRenderString32 (display, src_picture, mini_xft->u.ft.font, dest_picture, 0, 0,
+					 xs[k+1], ys[j+1], &glyph, 1);
+					 
+		  }
+	    }
+	  else
+	    {
+	      if (draw)
+		XftDrawString32 (draw, color, xft_font,
+				 x + PANGO_PIXELS (x_off + glyphs->glyphs[i].geometry.x_offset),
+				 y + PANGO_PIXELS (glyphs->glyphs[i].geometry.y_offset),
+				 &glyph, 1);
+	      else
+		XftRenderString32 (display, src_picture, xft_font->u.ft.font, dest_picture, 0, 0,
+				   x + PANGO_PIXELS (x_off + glyphs->glyphs[i].geometry.x_offset),
+				   y + PANGO_PIXELS (glyphs->glyphs[i].geometry.y_offset),
+				   &glyph, 1);
+	    }
+	}
+      
+      x_off += glyphs->glyphs[i].geometry.width;
+    }
+}
+
+
 /**
  * pango_xft_render:
  * @draw:    the XftDraw object.
@@ -218,88 +343,55 @@ pango_xft_render (XftDraw          *draw,
 		  gint              x,
 		  gint              y)
 {
-  PangoXftFont *xfont = PANGO_XFT_FONT (font);
-  Display *display;
-  int i;
-  int x_off = 0;
-
   g_return_if_fail (draw != NULL);
   g_return_if_fail (color != NULL);
-  g_return_if_fail (font != NULL);
+  g_return_if_fail (PANGO_XFT_IS_FONT (font));
   g_return_if_fail (glyphs != NULL);
   
-  /* Slow initial implementation. For speed, it should really
-   * collect the characters into runs, and draw multiple
-   * characters with each XftDrawString32 call.
-   */
+  pango_xft_real_render (NULL, None, None, draw, color, font, glyphs, x, y);
+}
+
+/**
+ * pango_xft_picture render:
+ * @draw:         an X display
+ * @src_picture:  the source picture to draw the string with
+ * @dest_picture: the destination picture to draw the strign onto
+ * @font:         the font in which to draw the string
+ * @glyphs:       the glyph string to draw
+ * @x:            the x position of start of string (in pixels)
+ * @y:            the y position of baseline (in pixels)
+ *
+ * Render a PangoGlyphString onto an Xrender picture object.
+ */
+void
+pango_xft_picture_render (Display          *display,
+			  Picture           src_picture,
+			  Picture           dest_picture,
+			  PangoFont        *font,
+			  PangoGlyphString *glyphs,
+			  gint              x,
+			  gint              y)
+{
+  g_return_if_fail (display != NULL);
+  g_return_if_fail (src_picture != None);
+  g_return_if_fail (dest_picture != None);
+  g_return_if_fail (PANGO_XFT_IS_FONT (font));
+  g_return_if_fail (glyphs != NULL);
   
-  _pango_xft_font_map_get_info (xfont->fontmap, &display, NULL);
-  
-  for (i=0; i<glyphs->num_glyphs; i++)
-    {
-      PangoGlyph glyph = glyphs->glyphs[i].glyph;
-
-      if (glyph)
-	{
-	  if (glyph & PANGO_XFT_UNKNOWN_FLAG)
-	    {
-	      char buf[5];
-	      int ys[3];
-	      int xs[3];
-	      int j, k;
-	      
-	      PangoFont *mini_font = get_mini_font (font);
-	      XftFont *mini_xft = ((PangoXftFont *)mini_font)->xft_font;
-	      FT_Face face = pango_xft_font_get_face (xfont->mini_font);
-      
-	      glyph &= ~PANGO_XFT_UNKNOWN_FLAG;
-
-	      ys[0] = y + PANGO_PIXELS (glyphs->glyphs[i].geometry.y_offset) - xfont->xft_font->ascent + (xfont->xft_font->ascent + xfont->xft_font->descent - xfont->mini_height * 2 - xfont->mini_pad * 5) / 2;
-	      ys[1] = ys[0] + 2 * xfont->mini_pad + xfont->mini_height;
-	      ys[2] = ys[1] + xfont->mini_height + xfont->mini_pad;
-
-	      xs[0] = x + PANGO_PIXELS (x_off + glyphs->glyphs[i].geometry.x_offset); 
-	      xs[1] = xs[0] + 2 * xfont->mini_pad;
-	      xs[2] = xs[1] + xfont->mini_width + xfont->mini_pad;
-	      
-	      draw_box (draw, color, xfont,
-			xs[0], ys[0],
-			xfont->mini_width * 2 + xfont->mini_pad * 5,
-			xfont->mini_height * 2 + xfont->mini_pad * 5);
-	      
-	      g_snprintf (buf, sizeof(buf), "%04X", glyph);
-
-	      for (j = 0; j < 2; j++)
-		for (k = 0; k < 2; k++)
-		  {
-		    XftChar32 glyph = FT_Get_Char_Index (face, buf[2*j + k]);
-		    XftDrawString32 (draw, color, mini_xft,
-				     xs[k+1], ys[j+1],
-				     &glyph, 1);
-		  }
-	    }
-	  else
-	    XftDrawString32 (draw, color, xfont->xft_font,
-			     x + PANGO_PIXELS (x_off + glyphs->glyphs[i].geometry.x_offset),
-			     y + PANGO_PIXELS (glyphs->glyphs[i].geometry.y_offset),
-			     &glyph, 1);
-	}
-      
-      x_off += glyphs->glyphs[i].geometry.width;
-    }
+  pango_xft_real_render (display, src_picture, dest_picture, NULL, NULL, font, glyphs, x, y);
 }
 
 static PangoFontMetrics *
 pango_xft_font_get_metrics (PangoFont        *font,
 			    PangoLanguage    *language)
 {
-  PangoXftFont *xfont = (PangoXftFont *)font;
   PangoFontMetrics *metrics = pango_font_metrics_new ();
+  XftFont *xft_font = pango_xft_font_get_font (font);
 
-  metrics->ascent = PANGO_SCALE * xfont->xft_font->ascent;
-  metrics->descent = PANGO_SCALE * xfont->xft_font->descent;
-  metrics->approximate_digit_width = PANGO_SCALE * xfont->xft_font->max_advance_width;
-  metrics->approximate_char_width = PANGO_SCALE * xfont->xft_font->max_advance_width;
+  metrics->ascent = PANGO_SCALE * xft_font->ascent;
+  metrics->descent = PANGO_SCALE * xft_font->descent;
+  metrics->approximate_digit_width = PANGO_SCALE * xft_font->max_advance_width;
+  metrics->approximate_char_width = PANGO_SCALE * xft_font->max_advance_width;
 
   return metrics;
 }
@@ -336,8 +428,11 @@ pango_xft_font_finalize (GObject *object)
     g_object_unref (xfont->ot_info);
 
   pango_font_description_free (xfont->description);
-  
-  XftFontClose (display, xfont->xft_font);
+
+  if (xfont->xft_font)
+    XftFontClose (display, xfont->xft_font);
+  else
+    XftPatternDestroy (xfont->font_pattern); /* If we opened the font it will own and destroy this pattern */
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -355,15 +450,17 @@ pango_xft_font_get_coverage (PangoFont     *font,
 			     PangoLanguage *language)
 {
   PangoXftFont *xfont = (PangoXftFont *)font;
-  const gchar *family = pango_font_description_get_family (xfont->description);
+  char *filename = NULL;
   FT_Face face;
   PangoCoverage *coverage;
   Display *display;
   int i;
   
   _pango_xft_font_map_get_info (xfont->fontmap, &display, NULL);
+
+  XftPatternGetString (xfont->font_pattern, XFT_FILE, 0, &filename);
   
-  coverage = _pango_xft_font_map_get_coverage (xfont->fontmap, family);
+  coverage = _pango_xft_font_map_get_coverage (xfont->fontmap, filename);
 
   if (coverage)
     return pango_coverage_ref (coverage);
@@ -381,7 +478,7 @@ pango_xft_font_get_coverage (PangoFont     *font,
 	pango_coverage_set (coverage, i, PANGO_COVERAGE_EXACT);
     }
 
-  _pango_xft_font_map_set_coverage (xfont->fontmap, family, coverage);
+  _pango_xft_font_map_set_coverage (xfont->fontmap, g_strdup (filename), coverage);
       
   return coverage;
 }
@@ -393,6 +490,7 @@ pango_xft_font_get_glyph_extents (PangoFont        *font,
 				  PangoRectangle   *logical_rect)
 {
   PangoXftFont *xfont = (PangoXftFont *)font;
+  XftFont *xft_font = pango_xft_font_get_font (font);
   XGlyphInfo extents;
   Display *display;
 
@@ -408,7 +506,7 @@ pango_xft_font_get_glyph_extents (PangoFont        *font,
       if (ink_rect)
 	{
 	  ink_rect->x = 0;
-	  ink_rect->y = PANGO_SCALE * (- xfont->xft_font->ascent + (xfont->xft_font->ascent + xfont->xft_font->descent - xfont->mini_height * 2 - xfont->mini_pad * 5) / 2);
+	  ink_rect->y = PANGO_SCALE * (- xft_font->ascent + (xft_font->ascent + xft_font->descent - xfont->mini_height * 2 - xfont->mini_pad * 5) / 2);
 	  ink_rect->width = PANGO_SCALE * (xfont->mini_width * 2 + xfont->mini_pad * 5);
 	  ink_rect->height = PANGO_SCALE * (xfont->mini_height * 2 + xfont->mini_pad * 5);
 	}
@@ -416,14 +514,14 @@ pango_xft_font_get_glyph_extents (PangoFont        *font,
       if (logical_rect)
 	{
 	  logical_rect->x = 0;
-	  logical_rect->y = - PANGO_SCALE * xfont->xft_font->ascent;
+	  logical_rect->y = - PANGO_SCALE * xft_font->ascent;
 	  logical_rect->width = PANGO_SCALE * (xfont->mini_width * 2 + xfont->mini_pad * 6);
-	  logical_rect->height = (xfont->xft_font->ascent + xfont->xft_font->descent) * PANGO_SCALE;
+	  logical_rect->height = (xft_font->ascent + xft_font->descent) * PANGO_SCALE;
 	}
     }
   else
     {
-      XftTextExtents32 (display, xfont->xft_font, &glyph, 1, &extents);
+      XftTextExtents32 (display, xft_font, &glyph, 1, &extents);
 
       if (ink_rect)
 	{
@@ -436,9 +534,9 @@ pango_xft_font_get_glyph_extents (PangoFont        *font,
       if (logical_rect)
 	{
 	  logical_rect->x = 0;
-	  logical_rect->y = - xfont->xft_font->ascent * PANGO_SCALE;
+	  logical_rect->y = - xft_font->ascent * PANGO_SCALE;
 	  logical_rect->width = extents.xOff * PANGO_SCALE;
-	  logical_rect->height = (xfont->xft_font->ascent + xfont->xft_font->descent) * PANGO_SCALE;
+	  logical_rect->height = (xft_font->ascent + xft_font->descent) * PANGO_SCALE;
 	}
     }
 }
@@ -481,11 +579,34 @@ XftFont *
 pango_xft_font_get_font (PangoFont *font)
 {
   PangoXftFont *xfont;
+  Display *display;
+  int screen;
+  FT_Face face;
+  FT_Error error;
+  int charmap;
 
   g_return_val_if_fail (PANGO_XFT_IS_FONT (font), NULL);
   
   xfont = PANGO_XFT_FONT (font);
 
+  if (xfont->xft_font == NULL)
+    {
+      _pango_xft_font_map_get_info (xfont->fontmap, &display, &screen);
+
+      xfont->xft_font = XftFontOpenPattern (display, xfont->font_pattern);
+
+      face = xfont->xft_font->u.ft.font->face;
+
+      /* There should be a unicode encoding, since we queried for it */
+      for (charmap = 0; charmap < face->num_charmaps; charmap++)
+	if (face->charmaps[charmap]->encoding == ft_encoding_unicode)
+	  break;
+  
+      g_assert (charmap != face->num_charmaps);
+  
+      error = FT_Set_Charmap(face, face->charmaps[charmap]);
+    }
+  
   return xfont->xft_font;
 }
 
@@ -512,7 +633,7 @@ pango_xft_font_get_display (PangoFont *font)
 }
 
 /**
- * pango_xft_get_unknown_glyph:
+ * pango_xft_font_get_unknown_glyph:
  * @font: a #PangoFont.
  * @wc: the Unicode character for which a glyph is needed.
  *
@@ -541,16 +662,16 @@ pango_xft_font_get_unknown_glyph (PangoFont *font,
 FT_Face
 pango_xft_font_get_face (PangoFont *font)
 {
-  PangoXftFont *xfont;
+  XftFont *xft_font;
 
   g_return_val_if_fail (PANGO_XFT_IS_FONT (font), NULL);
 
-  xfont = PANGO_XFT_FONT (font);
+  xft_font = pango_xft_font_get_font (font);
   
-  if (xfont->xft_font->core)
+  if (xft_font->core)
     return NULL;
   else
-    return xfont->xft_font->u.ft.font->face;
+    return xft_font->u.ft.font->face;
 }
 
 /**
